# -------------------------------------------------------------------------------
#   ----  www. r i d o l f a . e u ---
#
#
#### *Copyright 2018  Ridolfa.eu / Rolf S.*
#### *
#### * Licensed under the EUPL, Version 1.1 or – as soon they    *
####   will be approved by the European Commission - subsequent
####   versions of the EUPL (the "Licence");
#### * You may not use this work except in compliance with theLicence.
#### * You may obtain a copy of the Licence at:
####   https://joinup.ec.europa.eu/software/page/eupl
#
#   Rolf Schwerhoff:        RSf_220609 / V13
#   Project:                50 Solar
# ------------------------------------------------------------------------------
# Hot Water Boiler Configuration file 
# Solar Heater /  Micropython 
#


#-------------------------------------------------------------------------------
#   Miscellaneous
#-------------------------------------------------------------------------------

ERR_ERROR = -99  # General  error tag

#-------------------------------------------------------------------------------
#   SPI, LoRa
#-------------------------------------------------------------------------------

# SPI Bus Pico
LoRaSpi = {
    "Bus": 0,     # Hardwarebus
    "PinCS0": 1,
    "PinSCK": 2,
    "PinMOSI": 3,
    "PinMISO": 4,
    "Baudrate": 200000,  # Hz
    "Polarity": 0,
    "Phase": 0,
    }

# Future task: Replace with LoRa Implicit Header etc.
LoRaNode = {
    "F40CentralUnit": "A",
    "Boiler": "B"
    }

# Pin wiring LoRa device to Controller
LoraPins = {
    "Reset": 0,
    "NSS": LoRaSpi["PinCS0"],  # chip select
    "DIO0": 5  # receive irq
    }

# LoRa Radio Parameter
# Set radio parameters ( e.g  Transmit frequency, Signal bandwidth, Spreading factor, Coding rate)
# See register settings in the SX1276 data sheet

LoRaRadio = {
    "Freq": (868, 217, 0, 0),       # (868 MHz, RegFrMsb(0x06, RegFrMid(0x07), RegFrLsb(0x08)
    "RegConfig1": 0x72, # RegModemConfig1(0x1D
    "RegConfig2": 0x74, # RegModemConfig2(0x1E
    "RegConfig3": 0x04, # RegModemConfig3(0x26
    "TxPowerLevel": 5, # REG_PA_CONFIG
    "PreambleLength": 8, # REG_PREAMBLE_LSB , REG_PREAMBLE_LSB,
    "PowerPin": 1, # boost pin is 1, non-boost pin is 0
    "SyncWord": 0x12, # REG_SYNC_WORD
    }

#-------------------------------------------------------------------------------
#   Heater Control
#-------------------------------------------------------------------------------

# Pin wiring
Pin = {
    "PhaseL1": 9,  # Phase L1, PWM
    "PhaseL2": 8,  # Phase L2, on/off
    "PhaseL3": 7,  # Phase L3, on/off
    "NeoPixel": 13,  # Neopixel IN
    "Button2": 14,
    "ButtonAutomatic": 15,  # Automatic / manually  ( < 2 sec) or reset (> 10 sec) button
    "BoilerOnOff": 6,  # Current sensor input
    "DS18B20": 16,      # 1Wire bus, boiler temperature
    "DHT22": 17,         # Room temperature and humidity
    "LDR": 26   # Ambient light
}

NeopixSettings = {
    #  Count pixels clockwise, LedPhase2 ( Heizstab 2) starts with pixel 0
    "LedPhase1": [8, 9, 10, 11],
    "LedPhase2": 0,
    "LedPhase3": 1,
    "LedGridSolar": 6,
    "LedHostConnection": 3,
    "LedBoilerOnOff": 4,
    "LedNotUsed": [2, 5, 7],
    "DayBrightness": 100, # Brightness 0...255
    "NightBrightness": 1,
    "Red": (1, 0, 0),       # RGB colors
    "Green": (0, 1, 0),
    "Blue": (0, 0, 1),
    "White": (1, 1, 1),
    "Yellow": (1, 1, 0),
    "Cyan": (0, 1, 1),
    "Magenta": (1, 0, 1),
    "Black": (0, 0, 0),  # Led off
}

LDRSwitchPoint = {
    "Day": 10000,  # digital voltage value,0..65000
    "Night": 8000  # a hysteresis makes sense
}


# The following entries must match or be aligned with the associated CCU entries.

BASIC_TICK = 30  # sec, periodic timer, controls miscellaneous events
# sets  LedNoHostConnection and  solar power to zero.
HOST_CONNECTION_TIMEOUT = 180    # sec, recommended:  > 2 times of the lowest DataUpdate Time of the CCU
HEATER_POWER_MAX = 6000    # Watts, Default value in grid operation mode
L1_POWER_MIN = 200  # Watts, a minimal L1 (PWM) current is necessary for the Boiler On/Off recognition
SWITCH_POINT_HIGH = 4000  #  L1 + L2 + L3 on
SWITCH_POINT_LOW = 2000  # L1 + L2 or L3 on
ALIVE_SIGNAL_PREFIX = "a"  #  for test purposes  e.g. ESP32 TTGO  Sniffer

# Commands between Central control and Boiler control
Cmd = {
    "set_power": "P",
    "get_sonsordata": "D",
}

HeaterOpMode = {
    "GridPowered": "g",     # default
    "SolarPowered": "s",
    "HeaterOn": "o",
    "HeaterOff": "f",
}
