# -------------------------------------------------------------------------------
#   ----  www. r i d o l f a . e u ---
#
#
#### *Copyright 2018  Ridolfa.eu / Rolf S.*
#### *
#### * Licensed under the EUPL, Version 1.1 or – as soon they    *
####   will be approved by the European Commission - subsequent
####   versions of the EUPL (the "Licence");
#### * You may not use this work except in compliance with theLicence.
#### * You may obtain a copy of the Licence at:
####   https://joinup.ec.europa.eu/software/page/eupl
#
#   Rolf Schwerhoff:        RSf_220609
#   Project:                50 Solar
# ------------------------------------------------------------------------------
# Solar Plant Configuration file 


# -------------------------------------------------------------------------------
#   Miscellaneous
# -------------------------------------------------------------------------------
RaspiIP = {
    "AddrRemote":  "192.168.178.37",  # Raspi LAN IP, used if the Raspi is controlled by a remote machine
    # "AddrRemote":  "192.168.178.50",  # Raspi WLAN IP, used if the Raspi is controlled by a remote machine
    "PortRemote": 25000,
    "AddrLocal":  "localhost",  # WLAN only used if you have remote machine to control the Raspi
    "PortLocal": 8088,
}

# If there isn't enough exported power to supply all devices, a priority is needed.
# Currently only the heater is solar power controlled and the priority rules are not implemented
# 1  = high priority,  2  = low priority,
PowerPriority = {
    "WashingMachine": 1,    # just an example, not implemented
    "Heater": 2
}

STARTUP_TEXT = "-----  F40 Solar System  started  ------"

Logger = {
    "LogLevel": 20,  # DEBUG=10, INFO=20,  WARNING=30,  ERROR=40, CRITICAL=50
    "LogFile": 'f40solar.log',  # None if you want no logfile
    "FileSize": 524288,  # Logfilesize in bytes
    "FileBackups": 2,  # Number of Backup-Files
    "LogToConsole": True  # Print logs to console
}

Oled_StartScreen = {
    "TextA": 'SOLAR - BOILER',
    "TextB": 'Okt. 2022 / RolfS',
    "TextC": '--- Ridolfa.eu ---  2022',
    "Logo": 'logo_tierzucht.pm',
    # "Logo": 'logo_ridolfa.pm',
}

ERR_ERROR = "WarNix"   # -99  # General  error tag

# -------------------------------------------------------------------------------
#   SPI, LoRa
# -------------------------------------------------------------------------------
Spi = {
    "Bus": 0,     # Hardwarebus 0
    "PinSCK": 11,     # Pin 23
    "PinMOSI": 10,    # Pin 19
    "PinMISO": 9,     # Pin 21
    "PinCS0": 8,      # Pin 24
    "Baudrate": 200000,  # Hz
    "Mode": 1  # RasPi Mode (Polarity/Phase)  0=0/0, 1=0/1, 2=1/0, 3=1/1
    }

# Pin wirering LoRa device to RasPi
LoraPins = {
    "Reset": 23,   # Pin 16
    "NSS": Spi["PinCS0"],  # Pin 24    chip select
    "DIO0": 25  # Pin 22    receive irq
}

# Future task: Replace with LoRa Implicit Header etc.
LoRaNode = {
   "F40CentralUnit": "A",  # Raspi PV
   "Boiler": "B",  # Raspi Pico Betriebsbüro
   "TestTTGO": "C"  # TTGO ESP32 LoRa Oled
}

# Set radio parameters ( e.g.  Transmit frequency, Signal bandwidth, Spreading factor, Coding rate)
#  See Kellerweisheiten , P[13] and register settings in the SX1276 data sheet
LoRaRadio = {
    "Freq": (868, 217, 0, 0),       # (868 MHz, RegFrMsb(0x06, RegFrMid(0x07), RegFrLsb(0x08)
    "RegConfig1": 0x72,             # RegModemConfig1(0x1D
    "RegConfig2": 0x74,             # RegModemConfig2(0x1E
    "RegConfig3": 0x04,             # RegModemConfig3(0x26
    "TxPowerLevel": 5,                  # REG_PA_CONFIG
    "PreambleLength": 8,            # REG_PREAMBLE_LSB , REG_PREAMBLE_LSB,
    "PowerPin": 1,                      # boost pin is 1, non-boost pin is 0
    "SyncWord": 0x12,           # REG_SYNC_WORD
}

# -------------------------------------------------------------------------------
#   I2C, OLED SSD1306
# -------------------------------------------------------------------------------
I2c = {
    "Bus": 1,     # Hardwarebus 1
    "PinSDA": 2,     # Pin 3
    "PinSCL": 3,    # Pin 5
}

OLed = {
    "I2cAddr": 0x3c,
    "PixWidth": 128,
    "PixHeight": 64
}

# -------------------------------------------------------------------------------
#   Heater Control
# -------------------------------------------------------------------------------

# ## Data Update
# Due to the low data transmission and process time some events should have different execution times.
# If there is more than one action to be executed at the same time, only the action with the highest multiplier
# is performed. Example :
#  Base_Tick = 10sec,  ActionA: 2,  ActionB:  5
#     ActionA every 20 sec,     20, 40, 60, 80, 100, 120,....
#     ActionB every 50 sec.                         50, 100, 150,.....
#     After 100, 200, 300,...sec  only ActionB will be executed, AktionA is dropped

BASIC_TICK = 30  # sec,  timer intervall , initiates various actions
DataUpdate = {
     "SolarData": 2, # read solar plant registers
     "Power": 3,  # n, BASE_TICK multiplier
     "SensorData": 5,  # n, BASE_TICK multiplier
     "ArchiveData": 10  # n, BASE_TICK multiplier
}

ARCHIVE_FILE = "PvArchive.csv" # Long time data monitoring

NO_RESPONSE_MAX = 10    # number of consecutively allowed missing responses before a logger warning/error
AC_POWER_MIN = 300  # W Minimum exported power to activate the heater. The CCU min power should be greater-
AC_POWER_MAX = 6010  # W # not really necessary, the power is limited by the heater itself
ALIVE_SIGNAL_PREFIX = "a"  # for test purposes  e.g. ESP32 TTGO  Sniffer

HeaterOpMode = {
    "GridPowered": "g",     # default
    "SolarPowered": "s",
    "HeaterOn": "o",
    "HeaterOff": "f"
}

# Commands between Central control and Boiler control
Cmd = {         # copy to pico
    "set_power": "P",
    "get_sonsordata": "D"
}

# -------------------------------------------------------------------------------
#   Modbus , Registers
# -------------------------------------------------------------------------------
SolarPlantUnit = {
    'Inverter1': {
        'mb_id': 1,
        'regoffset': 0,
        'name': 'WR1',
    },
    'Inverter2': {
        'mb_id': 1,
        'regoffset': 0,
        'name': 'WR2',
    },
    'PowerMeter': {
        'mb_id': 2,
        'regoffset': 0,
        'name': 'POM',
    },
    'Battery': {
        'mb_id': 1,
        'regoffset': 0,
        'name': 'BAT',
    },
}

ModBusTCP = {
    'tcp_url': '192.168.178.17',
    'tcp_port': 1502,
    'unit_id':  SolarPlantUnit['PowerMeter']['mb_id'],
    'mb_timeout': 30
}

# Register data types
DT_INT16 = 1  # signed 16 bit integer
DT_SUNSSF = 2   # sunspec type ( = DT_INT16?)
DT_UINT16 = 3  # unsigned 16 bit integer
DT_UINT32 = 4
DT_UINT64 = 5
DT_FLOAT32 = 6  # IEE 754 float
DT_STR8 = 7  # 8 bit string
DT_STR16 = 8
DT_STR32 = 9

# ### Register lists  ######################################
# Only used registers are defined.
# See Kellerweisheiten and Project P50 for a complete list
# of registers e.g.SX1276 data sheet

# Register dictionaries
RegDicts = {
    "1": "RegsInverter",
    "2": "RegsMeter",
    "3": "RegsBattery"
}

# Technical Note – SunSpec Logging in SolarEdge Inverters, Version 2.4, December 2021
InverterStatus = {
    "1": ["I_STATUS_OFF", "Off"],
    "2": ["I_STATUS_SLEEPING", "Sleeping (auto-shutdown) – Night mode"],
    "3": ["I_STATUS_STARTING", "Grid Monitoring/wake-up"],
    "4": ["I_STATUS_MPPT", "Inverter is ON and producing power"],
    "5": ["I_STATUS_THROTTLED", "Production (curtailed)"],
    "6": ["I_STATUS_SHUTTING_DOWN", "Shutting down"],
    "7": ["I_STATUS_FAULT", "Fault"],
    "8": ["I_STATUS_STANDBY", "Maintenance/setup"]
}

""" Description of the register list items
Register adress = dict.key
0   Short name
1   Reg-Addr
2   Data bytes
3   Data type
4   Scalefactor register key
5   Description
6   Reg-Unit
7   Unitfactor
8   Display  unit
9   Display decimal places
10  Display text
"""

REG_INV_OFFSET = SolarPlantUnit['Inverter1']['regoffset']
RegsInverter = {
    "40072": ("I_AC_CurrentLxN", 40072, 1, DT_UINT16, "40076", "AC Total Current value", "A", 1, "A", 1, "Gesamtstrom"),
    "40073X": ("I_AC_CurrentLxN", 40073, 3, DT_UINT16, "40076", "AC Current Lx to N", "A", 1, "A", 1, "Lx-N Strom"),    # reads L1,L2,L3
    "40073": ("I_AC_CurrentL1N", 40073, 1, DT_UINT16, "40076", "AC Current L1 to N", "A", 1, "A", 1, "L1-N Strom"),
    "40074": ("I_AC_CurrentL2N", 40074, 1, DT_UINT16, "40076", "AC Current L2 to N", "A", 1, "A", 1, "L2-N Strom"),
    "40075": ("I_AC_CurrentL3N", 40075, 1, DT_UINT16, "40076", "AC Current L3 to N", "A", 1, "A", 1, "L3-N Strom"),
    "40076": ("I_AC_Current_SF", 40076, 1, DT_INT16, "Current Scale Factor"),

    "40079X": ("I_AC_VoltageLxN", 40079, 3, DT_UINT16, "40082", "AC Voltage Lx to N", "V", 1, "V", 1, "Lx-N Spannung"),  # reads L1,L2,L3
    "40079": ("I_AC_VoltageL1N", 40079, 1, DT_UINT16, "40082", "AC Voltage L1 to N", "V", 1, "V", 1, "L1-N Spannung"),
    "40080": ("I_AC_VoltageL2N", 40080, 1, DT_UINT16, "40082", "AC Voltage L2 to N", "V", 1, "V", 1, "L2-N Spannung"),
    "40081": ("I_AC_VoltageL3N", 40081, 1, DT_UINT16, "40082", "AC Voltage L3 to N", "V", 1, "V", 1, "L3-N Spannung"),
    "40082": ("I_AC_Voltage_SF", 40082, 1, DT_INT16, "Voltage Scale Factor"),
    "40083": ("I_AC_POWER", 40083, 1, DT_INT16, "40084", "AC Power", "W", 1, "W", 1, "AC Leistung"),
    "40084": ("I_AC_POWER_SF", 40084, 1, DT_INT16, "Power Scale Factor"),
    "40085": ("I_AC_Frequency", 40085, 1, DT_UINT16, "40086", "Frequency", "Hz", 1, "Hz", 1, "Frequenz"),
    "40086": ("I_AC_Frequency_SF", 40086, 1, DT_INT16, "Frequency Scale Factor"),

    "40096": ("I_DC_Current", 40096, 1, DT_UINT16, "40097", "DC Current", "A", 1, "A", 1, "PV Gleichstrom"),
    "40097": ("I_DC_Current_SF", 40097, 1, DT_INT16, "DC Current Scale Factor"),
    "40098": ("I_DC_Voltage", 40098, 1, DT_UINT16, "40099", "DC Voltage", "V", 1, "V", 1, "PV Gleichspannung"),
    "40099": ("I_DC_Voltage_SF", 40099, 1, DT_INT16, "DC Voltage Scale Factor"),
    "40100": ("I_DC_Power", 40100, 1, DT_INT16, "40101", "DC Power", "W", 1, "W", 1, "PV Gleichspannungsleistung"),
    "40101": ("I_DC_Power_SF", 40101, 1, DT_INT16, "DC Power Scale Factor"),
}

# Technical Note – SunSpec Logging in SolarEdge Inverters, Version 2.4, December 2021
REG_METER_OFFSET = SolarPlantUnit['PowerMeter']['regoffset'],
RegsMeter = {
    "40191X": ("M_AC_CurrentLxN", 40191, 3, DT_INT16, "40194", "AC Current Lx to N", "A", 1, "A", 1, "Lx-N Strom"),
    "40191": ("M_AC_CurrentL1N", 40191, 1, DT_INT16, "40194", "AC Current L1 to N", "A", 1, "A", 1, "Lx-N Strom"),
    "40192": ("M_AC_CurrentL2N", 40192, 1, DT_INT16, "40194", "AC Current L2 to N", "A", 1, "A", 1, "Lx-N Strom"),
    "40193": ("M_AC_CurrentL3N", 40193, 1, DT_INT16, "40194", "AC Current L3 to N", "A", 1, "A", 1, "Lx-N Strom"),
    "40194": ("M_AC_Current_SF", 40194, 1, DT_INT16, "Current Scale Factor"),
    # "40195": (40195, 1, DT_INT16, "40203", "L-N Voltage", "V", 2),
    "40196X": ("M_AC_VoltageLxN", 40196, 3, DT_INT16, "40203", "AC Voltage Lx to N", "V", 1, "V", 1, "Lx-N Spannung"),
    "40196": ("M_L1N Voltage", 40196, 1, DT_INT16, "40203", "AC Voltage L1 to N", "V", 1, "V", 1, "L1-N Spannung"),
    "40197": ("M_L2N Voltage", 40197, 1, DT_INT16, "40203", "AC Voltage L2 to N", "V", 1, "V", 1, "L1-N Spannung"),
    "40198": ("M_L3N Voltage", 40198, 1, DT_INT16, "40203", "AC Voltage L3 to N", "V", 1, "V", 1, "L1-N Spannung"),
    # "40199": (40199, 1, DT_INT16, "40203", "L-L Voltage", "V", 2),
    "40200": ("M_L1L2 Voltage", 40200, 1, DT_INT16, "40203", "AC Voltage L1 to L2", "V", 1, "V", 1, "L1-L2 Spannung"),
    "40201": ("M_L2L3 Voltage", 40201, 1, DT_INT16, "40203", "AC Voltage L2 to L3", "V", 1, "V", 1, "L2-L3 Spannung"),
    "40202": ("M_L3L1 Voltage", 40202, 1, DT_INT16, "40203", "AC Voltage L3 to L1", "V", 1, "V", 1, "L3-L1 Spannung"),
    "40203": ("M_AC_Voltage_SF", 40203, 1, DT_INT16, "Voltage Scale Factor"),
    "40204": ("M_AC_Frequency", 40204, 1, DT_INT16, "40205", "Frequency", "Hz", 1, "Hz", 1, "Frequenz"),
    "40205": ("M_AC_Frequency_SF", 40205, 1, DT_INT16, "Frequency Scale Factor"),
    "40206": ("M_AC_POWER", 40206, 1, DT_INT16, "40210", "AC Power", "W", 1, "W", 1, "AC Export(+) / Import(-) Leistung"),
    "40207": ("M_AC_POWER_L1", 40207, 1, DT_INT16, "40210", "AC Power L1", "W", 1, "W", 1, "AC Leistung L1"),
    "40208": ("M_AC_POWER_L2", 40208, 1, DT_INT16, "40210", "AC Power L2", "W", 1, "W", 1, "AC Leistung L2"),
    "40209": ("M_AC_POWER_L3", 40209, 1, DT_INT16, "40210", "AC Power L3", "W", 1, "W", 1, "AC Leistung L2"),
    "40210": ("M_AC_POWER_SF", 40210, 1, DT_INT16, "Power Scale Factor"),
    "40226": ("M_EXPORT_ENERGY", 40226, 2, DT_UINT32, "40242", "Total Exported Energy", "Wh", 0.001, "kWH", 3, "Gesamte exportierte Wirkenergie"),
    "40234": ("M_IMPORT_ENERGY", 40234, 2, DT_UINT32, "40242", "Total Imported Energy", "Wh", 0.001, "kWH", 3, "Gesamte importierte Wirkenergie"),
    "40242": ("M_ENERGY_SF", 40242, 1, DT_INT16, "Energy Scale Factor"),
}

# Technical Note – Power Control Protocol for SolarEdge Inverters,  Version 1.3, Nov. 2017
BatteryStatus = [
    "Off",
    "Standby",
    "Init",
    "Charge",
    "Discharge",
    "Fault",
    "Idle"
]

REG_BAT_OFFSET = SolarPlantUnit['Battery']['regoffset']
RegsBattery = {
            # 'M_DC_POWERx': ('M_DC_POWERx', 40226, 1, DT_INT16, 'M_DC_POWER_SF', 'Total DC Power', "W", "W", 1),
            # "c_manufacturer": (0xe100 , 16,  DT_STR16, "Manufacturer", "", 1),
            # "c_model": (0xe110 , 16, DT_STR16, "Model", "", 1),
            # "c_version": (0xe120 , 16, DT_STR16, "Version", "", 1),
            # "c_serialnumber": (0xe130 , 16,  DT_STR16, "Serial", "", 1),
            # "c_deviceaddress": (0xe140 , 1,DT_UINT16, "Modbus ID", "", 1),
            # "c_sunspec_did": (0xe141 , 1,DT_UINT16, "SunSpec DID", "", 1),
            #
            # "rated_energy": (0xe142 , 2,DT_FLOAT32, "Rated Energy", "Wh", 2),
            "57666": ("BAT_RATED_ENERGY", 57666, 2, DT_FLOAT32, "", "Rated Energy", "Wh", 0.001, "kWh", 2, "Gespeicherte Energie"),

            # "maximum_charge_continuous_power": (0xe144 , 2,DT_FLOAT32, "Maximum Charge Continuous Power", "W", 2),
            # "maximum_discharge_continuous_power": (0xe146 , 2,DT_FLOAT32, "Maximum Discharge Continuous Power", "W", 2),
            # "maximum_charge_peak_power": (0xe148 , 2,DT_FLOAT32, "Maximum Charge Peak Power", "W", 2),
            # "maximum_discharge_peak_power": (0xe14a , 2,DT_FLOAT32, "Maximum Discharge Peak Power", "W", 2),
            #
            # "average_temperature": (0xe16c , 2,DT_FLOAT32, "Average Temperature", "°C", 2),
            # "maximum_temperature": (0xe16e , 2,DT_FLOAT32, "Maximum Temperature", "°C", 2),

            # "57712 instantaneous_voltage": (0xe170 , 2,DT_FLOAT32, "Instantaneous Voltage", "V", 2),
            # "57714 instantaneous_current": (0xe172 , 2,DT_FLOAT32, "Instantaneous Current", "A", 2),
            # "57716 instantaneous_power": (0xe174 , 2,DT_FLOAT32, "Instantaneous Power", "W", 2),
            "57716": ("BAT_INST_POWER", 57716, 2, DT_FLOAT32, "", "Instantaneous Power", "W", 1, "W", 2, "Akku, Lade- (+)  oder Entladeleistung (-)"),
            #
            # "lifetime_export_energy_counter": (0xe176 , 4, DT_UINT64, "Total Exported Energy", "Wh", 2),
            # "lifetime_import_energy_counter": (0xe17A , 4, DT_UINT64, "Total Imported Energy", "Wh", 2),
            #
            # "maximum_energy": (0xe17e , 2,DT_FLOAT32, "Maximum Energy", "Wh", 2),
            # "available_energy": (0xe180 , 2,DT_FLOAT32, "Available Energy", "Wh", 2),
            "57728": ("BAT_AVAILABLE_ENERGY", 57728, 2, DT_FLOAT32, "", "Available Energy", "Wh", 0.001, "kWh", 2, "Akkuladung"),


            # "soh": (0xe182 , 2,DT_FLOAT32, "State of Health (SOH)", "%", 2),
            "57730": ("BAT_STATE OF HEALTH", 57730, 2, DT_FLOAT32, "", "State of Health (SOH)", "%", 1, "%", 2, "SOH"),
            # "soe": (0xe184 , 2,DT_FLOAT32, "State of Energy (SOE)", "%", 2),
            "57732": ("BAT_STATE OF ENERGY", 57732, 2, DT_FLOAT32, "", "State of Energy (SOE)", "%", 1, "%", 2, "SOE"),
            # "status": (0xe186 , 2, DT_UINT32, "Status", BatteryStatus, 2),
            # "status_internal": (0xe188 , 2, DT_UINT32, "Internal Status", BatteryStatus, 2),
            # "event_log": (0xe18a , 2, DT_UINT16, "Event Log", "", 2),
            # "event_log_internal": (0xe192 , 2, DT_UINT32, "Internal Event Log", "", 2),
}
