# -------------------------------------------------------------------------------
#   ----  www. r i d o l f a . e u ---
#
#
#### *Copyright 2018  Ridolfa.eu / Rolf S.*
#### *
#### * Licensed under the EUPL, Version 1.1 or – as soon they    *
####   will be approved by the European Commission - subsequent
####   versions of the EUPL (the "Licence");
#### * You may not use this work except in compliance with theLicence.
#### * You may obtain a copy of the Licence at:
####   https://joinup.ec.europa.eu/software/page/eupl
#
#   Rolf Schwerhoff:        RSf_220409
#   Project:                50 Solar
# ------------------------------------------------------------------------------

import config as cfg
from time import sleep
import datetime
from tools import SolarTools, OledTools, LoRaTools
from solarplant import SolarPlantRegisters
from heatercontrol import HeaterControl
from os import path


class F40Solar:
    def __init__(self):
        self.log_startup_text()
        self.filename = path.split(__file__)[1]
        self.filepath = path.split(__file__)[0]
        self.solartools = SolarTools()
        self.logger = self.init_logger()
        self.logger.info("File: {} ; {} ; {} ; {}".format(self.filename, cfg.STARTUP_TEXT, "--", "--"))
        text = "No pigpio connection, exit programm"
        try:
            self.pi, self.spi, self.i2c, filename, text = self.solartools.init_pigpio_spi_i2c(cfg.RaspiIP["AddrRemote"], cfg.RaspiIP["PortRemote"])  # remote pi
        except:
            try:
                self.pi, self.spi, self.i2c, filename, text = self.solartools.init_pigpio_spi_i2c(cfg.RaspiIP["AddrLocal"], cfg.RaspiIP["PortLocal"])  # local pi
            except:
                self.logger.error("File: {} ; {} ; {} ; {}".format(self.filename, text, "--", "--"))
                exit(0)
        self.oledtools = OledTools(self.pi, self.i2c)
        self.loratools = LoRaTools(self.pi, self.spi, self.logger)
        self.logger.info("File: {} ; {} ; {} ; {}".format(self.filename,  text, "--", "--"))

        self.solarregs = SolarPlantRegisters(logger=self.logger)
        sleep(1)  # give time to set up the inverters ModbusTCP connection

    def log_startup_text(self):
        with open(cfg.Logger['LogFile'], "a") as f:
            now = datetime.datetime.now()
            print(now.strftime("%Y-%m-%d %H:%M:%S"), cfg.STARTUP_TEXT, file=f)

    def init_logger(self):
        logger = self.solartools.init_logger(
            logconsole=cfg.Logger['LogToConsole'],
            logfile=cfg.Logger['LogFile'],
            loglevel=cfg.Logger['LogLevel'],
            filesize=cfg.Logger['FileSize'],
            filebackups=cfg.Logger['FileBackups']
        )
        return logger

    def run(self):
        HeaterControl(self.logger, self.oledtools, self.loratools, self.solarregs)


if __name__ == '__main__':
    F40Solar().run()
