# -------------------------------------------------------------------------------
#   ----  www. r i d o l f a . e u ---
#
#
#### *Copyright 2018  Ridolfa.eu / Rolf S.*
#### *
#### * Licensed under the EUPL, Version 1.1 or – as soon they    *
####   will be approved by the European Commission - subsequent
####   versions of the EUPL (the "Licence");
#### * You may not use this work except in compliance with theLicence.
#### * You may obtain a copy of the Licence at:
####   https://joinup.ec.europa.eu/software/page/eupl
#
#   Rolf Schwerhoff:        RSf_220609 
#   Project:                50 Solar
# ------------------------------------------------------------------------------
# Library copied from Adafriut and reduced to the needs
# of my SSD1306 OLED display

####################
# SPDX-FileCopyrightText: <text> 2018 Kattni Rembor, Melissa LeBlanc-Williams
# and Tony DiCola, for Adafruit Industries.
# https://github.com/adafruit/Adafruit_CircuitPython_framebuf.git
# Original file created by Damien P. George </text>
#
# SPDX-License-Identifier: MIT
#####################


import struct
# from os import path

MVLSB = 0  # Single bit displays (like SSD1306 OLED)


class FrameBuffer:
    """
    A minimized framebuffer version .
    For a fully functional version see Project p43
    """

    def __init__(self, buf, width, height, stride=None):
        self.buf = buf
        self.width = width
        self.height = height
        self.stride = stride
        self._font = None
        if self.stride is None:
            self.stride = width
        self.rotation = 0
        self.font_name = "font5x8.bin"

    def fill(self, color):
        """Fill the entire FrameBuffer with the specified color."""
        if color:
            fill = 0xFF
        else:
            fill = 0x00
        for i in range(len(self.buf)):  # pylint: disable=consider-using-enumerate
            self.buf[i] = fill

    def pixel(self, x, y, color=None):
        """
        If ``color`` is not given, get the color value of the specified pixel. If ``color`` is
        given, set the specified pixel to the given color.
        """
        if x < 0 or x >= self.width or y < 0 or y >= self.height:
            return None
        if color is None:
            return self.get_pixel(x, y)
        self.set_pixel(x, y, color)
        return None

    def set_pixel(self, x, y, color=1):
        """Set a given pixel to a color."""
        index = (y >> 3) * self.stride + x
        offset = y & 0x07
        self.buf[index] = (self.buf[index] & ~(0x01 << offset)) | ((color != 0) << offset)

    def get_pixel(self, x, y):
        """Get the color of a given pixel"""
        index = (y >> 3) * self.stride + x
        offset = y & 0x07
        return (self.buf[index] >> offset) & 0x01

    def fill_rect(self, x, y, width, height, color=1):
        """Draw a rectangle at the given location, size and color. """
        while height > 0:
            index = (y >> 3) * self.stride + x
            offset = y & 0x07
            for w_w in range(width):
                self.buf[index + w_w] = (
                    self.buf[index + w_w] & ~(0x01 << offset)
                ) | ((color != 0) << offset)
            y += 1
            height -= 1

    def line(self, x_0, y_0, x_1, y_1):
        """Bresenham's line algorithm"""
        d_x = abs(x_1 - x_0)
        d_y = abs(y_1 - y_0)
        x, y = x_0, y_0
        s_x = -1 if x_0 > x_1 else 1
        s_y = -1 if y_0 > y_1 else 1
        if d_x > d_y:
            err = d_x / 2.0
            while x != x_1:
                self.pixel(x, y)
                err -= d_y
                if err < 0:
                    y += s_y
                    err += d_x
                x += s_x
        else:
            err = d_y / 2.0
            while y != y_1:
                self.pixel(x, y)
                err -= d_x
                if err < 0:
                    x += s_x
                    err += d_y
                y += s_y
        self.pixel(x, y)

    def scroll(self, delta_x, delta_y):
        """shifts framebuf in x and y direction"""
        if delta_x < 0:
            shift_x = 0
            xend = self.width + delta_x
            dt_x = 1
        else:
            shift_x = self.width - 1
            xend = delta_x - 1
            dt_x = -1
        if delta_y < 0:
            y = 0
            yend = self.height + delta_y
            dt_y = 1
        else:
            y = self.height - 1
            yend = delta_y - 1
            dt_y = -1
        while y != yend:
            x = shift_x
            while x != xend:
                self.set_pixel(x, y, self.get_pixel(x - delta_x, y - delta_y))
                x += dt_x
            y += dt_y

    def text(self, string, x, y, size=1):
        """
        Place text on the screen in variables sizes. Breaks on \n to next line.
        Does not break on line going off screen.
        """
        frame_width = self.width
        frame_height = self.height
        for chunk in string.split("\n"):
            width, height = self.get_font_size()
            for i, char in enumerate(chunk):
                char_x = x + (i * (width + 1)) * size
                if (char_x + (width * size) > 0
                        and char_x < frame_width
                        and y + (height * size) > 0
                        and y < frame_height):
                    self.draw_char(char, char_x, y, width, height, size=size)
            y += height * size

    def get_font_size(self):

        with open(self.font_name, "rb") as self._font:
            return struct.unpack("BB", self._font.read(2))

    def draw_char(self, char, x, y, font_width, font_height, size=1):
        """Draw one character at position (x,y) to a framebuffer"""
        return

        with open(self.font_name, "rb") as self._font:
            size = max(size, 1)
            # Go through each column of the character.
            for char_x in range(font_width):
                # Grab the byte for the current column of font data.
                self._font.seek(2 + (ord(char) * font_width) + char_x)
                try:
                    line = struct.unpack("B", self._font.read(1))[0]
                except RuntimeError:
                    continue  # maybe character isn't there? go to next
                # Go through each row in the column byte.
                for char_y in range(font_height):
                    # Draw a pixel for each bit that's flipped on.
                    if (line >> char_y) & 0x1:
                        self.fill_rect(x + char_x * size, y + char_y * size, size, size)
