# -------------------------------------------------------------------------------
#   ----  www. r i d o l f a . e u ---
#
#
#### *Copyright 2018  Ridolfa.eu / Rolf S.*
#### *
#### * Licensed under the EUPL, Version 1.1 or – as soon they    *
####   will be approved by the European Commission - subsequent
####   versions of the EUPL (the "Licence");
#### * You may not use this work except in compliance with theLicence.
#### * You may obtain a copy of the Licence at:
####   https://joinup.ec.europa.eu/software/page/eupl
#
#   Rolf Schwerhoff:        RSf_220609
#   Project:                50 Solar
# ------------------------------------------------------------------------------
# TTGO Version, A sniffer for project P50 to receive / send messages

from time import sleep
from machine import Timer
import config as cfg
from tools import LoRaTools


class BoilerControl:

    def __init__(self, spi, oledtools):
        self.lora_id = cfg.LoRaNode["Sniffer"] + cfg.LoRaNode["F40CentralUnit"]
        self.rx_msg = "-"
        self.no_command_count = 1
        self.pwm_power = 0
        self.opmode_solar = True  # False = Grid Mode
        self.alive_count = 0
        self.stack = ["----", "----", "----"]  # Oled scroll stack
        self.oledtools = oledtools
        self.oledtools.oled_init()
        self.oledtools.oled_startup_screen()
        self.loratools = LoRaTools(spi)
        self.oledtools.oled_text("-- RolfS --", 0, 10)
        self.loratools.init_recv_mode(self.receive)
        self.tim = Timer(0)
        self.tim.init(period=1000 * cfg.SEND_ALIVE_SIGNAL, mode=Timer.PERIODIC, callback=self.send_alive)  # msec

    def send(self, tx_data):
        """
        Sends a message , (re)enables the reception mode
        and set the status bytes
        """
        # Add Transmitter and Receiver ID
        text = "TX " + self.lora_id + tx_data
        print(42, text)
        self.oledtools.oled_scroll(self.stack, text)
        sleep(0.5)
        self.loratools.send(tx_data)
        self.loratools.set_recv_mode()

    def receive(self, pin_irq):
        """ This function is triggered ( DIO0 IRQ) by a new LoRa  message """
        self.process_rxdata(self.loratools.get_recv_data())

    def process_rxdata(self, rx_data):
        """
         Processes the received message 
         and sends an acknowledgement back
          """
        packet_rssi_strength = str(self.loratools.get_packet_rssi_strength())
        text = "RX {} {}".format(rx_data, packet_rssi_strength)
        self.oledtools.oled_scroll(self.stack, text)
        print(60, text)

    def send_alive(self, timerobj):
        """ Alive signal for everyone """
        text = "a TTGO_" + str(self.alive_count)
        self.send(text)
        self.alive_count += 1
        if self.alive_count >= 100:
            self.alive_count = 0


if __name__ == "__main__":
    exec(open("boiler_run.py").read())
